@@ -10,7 +10,6 @@ Redistribution and use in source and binary forms, with or without modification,
 
     * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-    * Neither the author nor the names of any contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
@@ -441,7 +440,6 @@ static void kf_work(
         const celt_int16 * factors,
         const kiss_fft_state *st,
         int N,
-        int s2,
         int m2
         )
 {
@@ -449,7 +447,7 @@ static void kf_work(
     const int m=*factors++; /* stage's fft length/p */
     /*printf ("fft %d %d %d %d %d %d %d\n", p*m, m, p, s2, fstride*in_stride, N, m2);*/
     if (m!=1) 
-        kf_work( Fout , f, fstride*p, in_stride, factors,st, N*p, fstride*in_stride, m);
+        kf_work( Fout , f, fstride*p, in_stride, factors,st, N*p, m);
 
     /* Compensate for longer twiddles table (when sharing) */
     if (st->shift>0)
@@ -473,7 +471,6 @@ static void ki_work(
              const celt_int16 * factors,
              const kiss_fft_state *st,
              int N,
-             int s2,
              int m2
             )
 {
@@ -481,7 +478,7 @@ static void ki_work(
    const int m=*factors++; /* stage's fft length/p */
    /*printf ("fft %d %d %d %d %d %d %d\n", p*m, m, p, s2, fstride*in_stride, N, m2);*/
    if (m!=1) 
-      ki_work( Fout , f, fstride*p, in_stride, factors,st, N*p, fstride*in_stride, m);
+      ki_work( Fout , f, fstride*p, in_stride, factors,st, N*p, m);
 
    /* Compensate for longer twiddles table (when sharing) */
    if (st->shift>0)
@@ -497,7 +494,7 @@ static void ki_work(
 }
 
 
-#ifndef STATIC_MODES
+#ifdef CUSTOM_MODES
 
 static
 void compute_bitrev_table(
@@ -647,13 +644,16 @@ kiss_fft_state *kiss_fft_alloc(int nfft,void * mem,size_t * lenmem )
 
 void kiss_fft_free(const kiss_fft_state *cfg)
 {
-   celt_free((celt_int16*)cfg->bitrev);
-   if (cfg->shift < 0)
-      celt_free((kiss_twiddle_cpx*)cfg->twiddles);
-   celt_free((kiss_fft_state*)cfg);
+   if (cfg)
+   {
+      celt_free((celt_int16*)cfg->bitrev);
+      if (cfg->shift < 0)
+         celt_free((kiss_twiddle_cpx*)cfg->twiddles);
+      celt_free((kiss_fft_state*)cfg);
+   }
 }
 
-#endif /* STATIC_MODES */
+#endif /* CUSTOM_MODES */
 
 static void kiss_fft_stride(const kiss_fft_state *st,const kiss_fft_cpx *fin,kiss_fft_cpx *fout,int in_stride)
 {
@@ -668,7 +668,7 @@ static void kiss_fft_stride(const kiss_fft_state *st,const kiss_fft_cpx *fin,kis
        fout[st->bitrev[i]].i *= st->scale;
 #endif
     }
-    kf_work( fout, fin, 1,in_stride, st->factors,st, 1, in_stride, 1);
+    kf_work( fout, fin, 1,in_stride, st->factors,st, 1, 1);
 }
 
 void kiss_fft(const kiss_fft_state *cfg,const kiss_fft_cpx *fin,kiss_fft_cpx *fout)
@@ -683,7 +683,7 @@ static void kiss_ifft_stride(const kiss_fft_state *st,const kiss_fft_cpx *fin,ki
    /* Bit-reverse the input */
    for (i=0;i<st->nfft;i++)
       fout[st->bitrev[i]] = fin[i];
-   ki_work( fout, fin, 1,in_stride, st->factors,st, 1, in_stride, 1);
+   ki_work( fout, fin, 1,in_stride, st->factors,st, 1, 1);
 }
 
 void kiss_ifft(const kiss_fft_state *cfg,const kiss_fft_cpx *fin,kiss_fft_cpx *fout)
