@@ -14,10 +14,6 @@
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
    
-   - Neither the name of the Xiph.org Foundation nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-   
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -315,7 +311,7 @@ static CELTDecoder *process_header(ogg_packet *op, celt_int32 enh_enabled, celt_
    
    *channels = header.nb_channels;
    *overlap=header.overlap;
-   st = celt_decoder_create(*mode, header.nb_channels, NULL);
+   st = celt_decoder_create_custom(*mode, header.nb_channels, NULL);
    if (!st)
    {
       fprintf (stderr, "Decoder initialization failed.\n");
@@ -533,9 +529,9 @@ int main(int argc, char **argv)
          /*printf ("page granulepos: %d %d %d\n", skip_samples, page_nb_packets, (int)page_granule);*/
          last_granule = page_granule;
          /*Extract all available packets*/
-         while (!eos && ogg_stream_packetout(&os, &op) == 1 && op.bytes>=8)
+         while (!eos && ogg_stream_packetout(&os, &op) == 1)
          {
-	    if (!memcmp(op.packet, "CELT    ", 8)) {
+	    if (op.bytes>=8 && !memcmp(op.packet, "CELT    ", 8)) {
 	       celt_serialno = os.serialno;
 	    }
 	    if (celt_serialno == -1 || os.serialno != celt_serialno)
@@ -577,9 +573,9 @@ int main(int argc, char **argv)
                   /*for (i=0;i<frame_size*channels;i++)
                     printf ("%d\n", (int)output[i]);*/
 
-                  if (ret!=0)
+                  if (ret<0)
                   {
-                     fprintf (stderr, "Decoding error: corrupted stream?\n");
+                     fprintf (stderr, "Decoding error: %s\n", celt_strerror(ret));
                      break;
                   }
 
