@@ -14,10 +14,6 @@
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
    
-   - Neither the name of the Xiph.org Foundation nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-   
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -52,6 +48,10 @@
 #include <string.h>
 #include <time.h>
 
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
+
 #include "celt.h"
 #include "celt_header.h"
 #include <ogg/ogg.h>
@@ -88,9 +88,9 @@ int oe_write_page(ogg_page *page, FILE *fp)
 /* Convert input audio bits, endians and channels */
 static int read_samples(FILE *fin,int frame_size, int bits, int channels, int lsb, short * input, char *buff, celt_int32 *size)
 {   
-   unsigned char in[MAX_FRAME_SIZE*2];
+   short s[MAX_FRAME_SIZE];
+   unsigned char *in = (unsigned char*)s;
    int i;
-   short *s;
    int nb_read;
 
    if (size && *size<=0)
@@ -116,7 +116,6 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
    if (nb_read==0)
       return 0;
 
-   s=(short*)in;
    if(bits==8)
    {
       /* Convert 8->16 bits */
@@ -140,7 +139,7 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
    /* copy to float input buffer */
    for (i=0;i<frame_size*channels;i++)
    {
-      input[i]=(short)s[i];
+      input[i]=s[i];
    }
 
    for (i=nb_read*channels;i<frame_size*channels;i++)
@@ -218,8 +217,8 @@ void usage(void)
    printf (" --cbr              Use constant bitrate encoding\n");
    printf (" --comp n           Encoding complexity (0-10)\n");
    printf (" --framesize n      Frame size (Default: 960)\n");
-   printf (" --noltp            Do not use long-term prediction\n");
-   printf (" --independent      Encode frames independently (implies noltp)\n");
+   printf (" --nopf             Do not use the prefilter/postfilter\n");
+   printf (" --independent      Encode frames independently (implies nopf)\n");
    printf (" --skeleton         Outputs ogg skeleton metadata (may cause incompatibilities)\n");
    printf (" --comment          Add the given string as an extra comment. This may be\n");
    printf ("                     used multiple times\n");
@@ -265,7 +264,7 @@ int main(int argc, char **argv)
       {"cbr",no_argument,NULL, 0},
       {"cvbr",no_argument,NULL, 0},
       {"comp", required_argument, NULL, 0},
-      {"noltp", no_argument, NULL, 0},
+      {"nopf", no_argument, NULL, 0},
       {"independent", no_argument, NULL, 0},
       {"framesize", required_argument, NULL, 0},
       {"skeleton",no_argument,NULL, 0},
@@ -378,7 +377,7 @@ int main(int argc, char **argv)
          } else if (strcmp(long_options[option_index].name,"framesize")==0)
          {
             frame_size=atoi (optarg);
-         } else if (strcmp(long_options[option_index].name,"noltp")==0)
+         } else if (strcmp(long_options[option_index].name,"nopf")==0)
          {
             if (prediction>1)
               prediction=1;
@@ -478,23 +477,7 @@ int main(int argc, char **argv)
      else
        bitrate=128.0;
      
-   bytes_per_packet = (bitrate*1000*frame_size/rate+4)/8;
-   
-   if (bytes_per_packet < 8) {
-      bytes_per_packet=8;
-      fprintf (stderr, "Warning: Requested bitrate (%0.3fkbit/sec) is too low. Setting CELT to 8 bytes/frame.\n",bitrate);
-   } else if (bytes_per_packet > MAX_FRAME_BYTES) {
-      bytes_per_packet=MAX_FRAME_BYTES;
-      fprintf (stderr, "Warning: Requested bitrate (%0.3fkbit/sec) is too high. Setting CELT to %d bytes/frame.\n",bitrate,MAX_FRAME_BYTES);      
-   }
-
-   if (with_cbr)
-   {
-     bitrate = ((rate/(float)frame_size)*8*bytes_per_packet)/1000.0;
-   } else {
-     /*In VBR mode the bytes_per_packet argument becomes a hard maximum. 3x the average rate is just a random choice.*/
-     bytes_per_packet=IMIN(bytes_per_packet*3,MAX_FRAME_BYTES);
-   }
+   bytes_per_packet = MAX_FRAME_BYTES;
    
    mode = celt_mode_create(rate, frame_size, NULL);
    if (!mode)
@@ -507,7 +490,7 @@ int main(int argc, char **argv)
 
    /*celt_mode_info(mode, CELT_GET_FRAME_SIZE, &frame_size);*/
    
-   celt_header_init(&header, mode, chan);
+   celt_header_init(&header, mode, frame_size, chan);
    header.nb_channels = chan;
 
    {
@@ -524,12 +507,19 @@ int main(int argc, char **argv)
    }
 
    /*Initialize CELT encoder*/
-   st = celt_encoder_create(mode, chan, NULL);
+   st = celt_encoder_create_custom(mode, chan, NULL);
 
+   {
+      int tmp = (bitrate*1000);
+      if (celt_encoder_ctl(st, CELT_SET_BITRATE(tmp)) != CELT_OK)
+      {
+         fprintf (stderr, "bitrate request failed\n");
+         return 1;
+      }
+   }
    if (!with_cbr)
    {
-     int tmp = (bitrate*1000);
-     if (celt_encoder_ctl(st, CELT_SET_VBR_RATE(tmp)) != CELT_OK)
+     if (celt_encoder_ctl(st, CELT_SET_VBR(1)) != CELT_OK)
      {
         fprintf (stderr, "VBR request failed\n");
         return 1;
